#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##################################
# Standard library imports
##################################
import argparse
from multiprocessing import cpu_count

##################################
# Third party imports
##################################
import numpy as np
import pandas as pd
from concurrent.futures import ProcessPoolExecutor

##################################
# Local imports
##################################
from gdelt.helpers import shaper

cores = cpu_count()
e = ProcessPoolExecutor(max_workers=cores)


def shaper(row):
    """
    Parallel function to create shapely points
    from latitude/longitude pair in dataframe

    Parameters
    ----------
    row : pandas or dask dataframe row
        Row containing latitude and longitude variables and data

    Returns
    -------
    shapely point
        Shapely spatial object for geoprocessing in Python.
    """
    try:
        from shapely.geometry import Point
    except:
        raise ImportError('You need to install shapely to use this feature.')

    try:
        import fiona
    except:
        raise ImportError('You need to install fiona to use this feature.')
    try:
        import geopandas
    except:
        raise ImportError('You need to install geopandas to use this feature.')
    geometry = Point(row['ActionGeo_Long'], row['ActionGeo_Lat'])
    return geometry


def call_apply_fn(df):
    return df.apply(shaper, axis=1)


def parallelize_dataframe(df):
    """Applying function"""
    df_split = np.array_split(df, cores * 2)
    finaldf = pd.concat(list(e.map(call_apply_fn, df_split)))
    return finaldf


#
# if __name__ == '__main__':
#     import sys
#     parallelize_dataframe(sys.argv[1:])
if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Parallelize pandas apply function')
    parser.add_argument('--df', metavar='pandas.core.frame.DataFrame', required=True,
                        help='The target dataframe to apply the function.')
    parser.add_argument('--func', metavar='function', required=False,
                        help='The function to be applied to the dataframe.')
    args = parser.parse_args()
    # model_schema(df=args.df, func=args.func)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##################################
# Standard library imports
##################################
from io import BytesIO

##################################
# Third party imports
##################################
import pandas as pd
import requests


#######################################
# Headers for GDELT 1.0 Events
#######################################

def events1Heads():
    """

    :rtype: dateframe
    """
    eventsDbHeaders = pd.read_csv(
        BytesIO(
            requests.get(
                ("https://raw.githubusercontent.com/linwoodc3/gdeltPyR"
                 "/master/utils/schema_csvs/"
                 "GDELT_1.0_event_Column_Labels_Header_Row_Sep2016.tsv")
            ).content), delimiter='\t', usecols=['tableId'])
    return eventsDbHeaders.tableId.tolist()


#######################################
# Headers for GDELT 2.0 Events
#######################################

def events2Heads():
    eventsDbHeaders = pd.read_csv(
        BytesIO(
            requests.get(
                ("https://raw.githubusercontent.com/linwoodc3/gdeltPyR/"
                 "master/utils/schema_csvs/"
                 "GDELT_2.0_Events_Column_Labels_Header_Row_Sep2016.csv")
            ).content), delimiter=',', usecols=['tableId', 'dataType',
                                                'Description'])
    return eventsDbHeaders.tableId.tolist()


#######################################
# Headers for GDELT 2.0 Mentions
#######################################

def mentionsHeads():
    eventsMentionsHeaders = pd.read_csv(
        BytesIO(
            requests.get(
                ("https://raw.githubusercontent.com/linwoodc3/gdeltPyR/master/"
                 "utils/schema_csvs/"
                 "GDELT_2.0_eventMentions_Column_Labels_Header_"
                 "Row_Sep2016.tsv")
            ).content), delimiter='\t', usecols=['tableId', 'dataType',
                                                 'Description'])
    return eventsMentionsHeaders.tableId.tolist()


#######################################
# Headers for GDELT 2.0 GKG
#######################################

def gkgHeads():
    gkgHeaders = pd.read_csv(
        BytesIO(
            requests.get(
                ("https://raw.githubusercontent.com/linwoodc3/gdeltPyR/"
                 "master/utils/schema_csvs/"
                 "GDELT_2.0_gdeltKnowledgeGraph_Column_Labels_Header_Row_"
                 "Sep2016.tsv")
            ).content), delimiter='\t', usecols=['tableId', 'dataType',
                                                 'Description'])
    return gkgHeaders.tableId.tolist()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##############################
# Standard Library Imports
##############################
import re
import datetime


##############################
# Third party Library Imports
##############################
from dateutil.parser import parse

##############################
# Filter functions for dataframes
##############################

def cameos(x, codes):
    try:
        return codes['Description'][x]
    except:
        return "No Description returned for CAMEO code {0}".format(x)


def shaper(row):
    """
    Parallel function to create shapely points
    from latitude/longitude pair in dataframe

    Parameters
    ----------
    row : pandas or dask dataframe row
        Row containing latitude and longitude variables and data

    Returns
    -------
    shapely point
        Shapely spatial object for geoprocessing in Python.
    """
    try:
        from shapely.geometry import Point
    except:
        raise ImportError('You need to install shapely to use this feature.')

    try:
        import fiona
    except:
        raise ImportError('You need to install fiona to use this feature.')
    try:
        import geopandas
    except:
        raise ImportError('You need to install geopandas to use this feature.')
    geometry = Point(row['ActionGeo_Long'], row['ActionGeo_Lat'])
    return geometry


def rooturl(row):
    """Finds the root url of a news article"""
    s = row['SOURCEURL']
    r = re.compile('(?<=http://)([A-Za-z0-9\.]+)(?=/)')
    r2 = re.compile('(?<=https://)([A-Za-z0-9\.-]+)(?=/)')

    if r.search(s):
        return r.search(s).group()
    elif r2.search(s):
        return r2.search(s).group()

def testdate(dateString):
    """Test dates of different formats"""
    if len(dateString) == 4:
        comp = datetime.datetime.strptime(dateString, '%Y')
    elif len(dateString) == 6:
        comp = datetime.datetime.strptime(dateString, '%Y%m')
    else:
        comp = parse(dateString)
    return comp


#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##################################
# Standard library imports
##################################
import re
import zipfile
from io import BytesIO
import warnings, datetime

##################################
# Third party imports
##################################
import pandas as pd
import requests

##################################
# Local imports
##################################

def downloadAndExtract(gdeltUrl):
    """Downloads and extracts GDELT zips without saving to disk"""

    response = requests.get(gdeltUrl, stream=True)
    zipdata = BytesIO()
    zipdata.write(response.content)
    gdelt_zipfile = zipfile.ZipFile(zipdata, 'r')
    name = re.search('(([\d]{4,}).*)',
                     gdelt_zipfile.namelist()[0]).group().replace('.zip', "")
    data = gdelt_zipfile.read(name)
    gdelt_zipfile.close()
    del zipdata, gdelt_zipfile, name, response
    return pd.read_csv(BytesIO(data), delimiter='\t', header=None)


def normalpull(url,table=None):
    """When single string url, just download it"""
    r = requests.get(url, timeout=5)
    # print(len(r.content))
    r.raise_for_status()

    # print (multiprocessing.Process(name=multiprocessing.current_process().name).is_alive())
    try:
        buffer = BytesIO(r.content)
        # print(buffer.seek())

        if table == 'events':
            frame = pd.read_csv(buffer, compression='zip', sep='\t',
                                header=None, warn_bad_lines=False,
                                dtype={26: 'str', 27: 'str', 28: 'str'},
                                parse_dates=[1, 2])
            # print(frame.shape)
        elif table == 'gkg':
            frame = pd.read_csv(buffer, compression='zip', sep='\t',
                                parse_dates=['DATE'], warn_bad_lines=False)

        else:

            frame = pd.read_csv(buffer, compression='zip', sep='\t',
                                header=None, warn_bad_lines=False)
        end = datetime.datetime.now() - start
        # print ("{0} with id {1} finished processing in {2}".format(proc_name,proc,end))
        buffer.flush()
        buffer.close()
        # print(frame.shape)
        return frame

    except:
        try:
            message = "GDELT did not return data for date time " \
                      "{0}".format(re.search('[0-9]{4,18}', url).group())
            warnings.warn(message)
        except:
            message = "No data returned for {0}".format(r.url)
            warnings.warn(message)


#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##################################
# Standard Library imports
##################################
import datetime

############################
# Third Party imports
#############################
import numpy as np
import pandas as pd
from dateutil.parser import parse

################################
# Local Imports
################################
from gdelt.multipdf import parallelize_dataframe
from gdelt.helpers import testdate


def vectorizer(function, dateArray):
    helper = np.vectorize(function)

    final = helper(dateArray.tolist()).tolist()

    if isinstance(final, list):

        final = list(set(final))
    elif isinstance(final, str):
        final = final
    else:
        pass

    return final


# Finds the urls from an array of dates


def urlFinder(dataframe, targetDate, col):
    return dataframe[col][dataframe[col].str.contains(targetDate)]


def vectorizedUrlFinder(function, urlList, frame):
    helper = np.vectorize(function)
    return pd.concat(helper(urlList, frame).tolist())


def downloadVectorizer(function, urlList):
    """Vectorized function to download urls"""
    helper = np.vectorize(function)
    return pd.concat(helper(urlList).tolist()).reset_index(drop=True)

def urlBuilder(dateString, version, table='events'):
    """
    Takes date string from gdeltRange string and creates GDELT urls


    Parameters
    ------------

    table types:
                * events and mentions (default)
                * gkg
                * mentions only
                :type version: object
                :param dateString:
                :param version:
                :param table:
    """

    if version == 2:
        base = "http://data.gdeltproject.org/gdeltv2/"

    if version == 1:
        base = "http://data.gdeltproject.org/"

    if table == "events":
        if version == 1:
            base += 'events/'
        caboose = ".export.CSV.zip"
    elif table == "mentions":
        caboose = ".mentions.CSV.zip"
    elif table == "gkg":
        if version == 1:
            base += 'gkg/'
            if isinstance(dateString,str):
                comp = testdate(dateString)
                if comp < parse('2013 Apr 1'):
                    raise Exception('GDELT 1.0 Global Knowledge Graph requires dates greater'
                                    ' than or equal to April 1 2013')
            elif isinstance(dateString, list) or isinstance(dateString,
                                                          np.ndarray):
                if not (np.all(list(
                        map(
                            lambda x: x > parse('2013 04 01'), list(
                                map(
                                    testdate, dateString)))))):
                    raise Exception('GDELT 1.0 Global Knowledge Graph requires dates greater'
                                    ' than or equal to April 1 2013')

            # if len(dateString) == 4:
            #     comp = datetime.datetime.strptime(dateString, '%Y')
            # elif len(dateString) == 6:
            #     comp = datetime.datetime.strptime(dateString, '%Y%m')
            # else:
            #     comp = parse(dateString)
            # if comp < parse('2013 Apr 1'):
            #     raise Exception('GDELT 1.0 Global Knowledge Graph requires dates greater'
            #                     ' than or equal to April 1 2013')


        caboose = ".gkg.csv.zip"
    else:
        raise ValueError('You entered an incorrect GDELT table type.'
                         ' Choose between \"events\",\"mentions\",'
                         'and \"gkg\".')

    # if version == 2:
    #     base = "http://data.gdeltproject.org/gdeltv2/"
    #
    # if version == 1:
    #     base = "http://data.gdeltproject.org/"
    #
    # if isinstance(dateString,str):
    #     if len(dateString) == 4:
    #         comp = datetime.datetime.strptime(dateString, '%Y')
    #     elif len(dateString) == 6:
    #         comp = datetime.datetime.strptime(dateString, '%Y%m')
    #     else:
    #         comp = parse(dateString)
    #
    # if table == "events":
    #     if version ==1 and isinstance(dateString,str):
    #         base += 'events/'
    #         try:
    #             if comp<parse('2006 Jan 1'):
    #                 caboose = ".zip"
    #         except:
    #             try:
    #                 if comp < parse('2006 Jan 1'):
    #                     caboose='.zip'
    #             except :
    #                 raise Exception('Invalid date entry')
    #     else:
    #         caboose = ".export.CSV.zip"
    #
    # elif table == "mentions":
    #     caboose = ".mentions.CSV.zip"
    # # elif table == "gkg":
    # #     print(dateString)
    # #     if version == 1:
    # #         if comp< parse('2013 Apr 01'):
    # #             raise Exception('GDELT 1.0 Global Knowledge Graph requires dates greater than or equal to April 1 2013.')
    # #         base += 'gkg/'
    # #     elif version == 2:
    # #         if comp < parse('2015 Feb 18 11:30'):
    # #             raise Exception('GDELT 2.0 Global Knowledge Graph requires dates greater than or equal to Feb 18 2015 11 pm.')
    # #     caboose = ".gkg.csv.zip"
    # else:
    #     raise ValueError('You entered an incorrect GDELT table type.'
    #                      ' Choose between \"events\",\"mentions\",'
    #                      'and \"gkg\".')


    if isinstance(dateString, list) is True or isinstance(dateString,
                                                          np.ndarray) is True:
        # print("This is before any changes {}".format(dateString))
        newdate = []
        olddateString = dateString
        date=dateString
        for l in date:
            if len(l) == 4:
                test = (str(datetime.datetime.strptime(l, '%Y')))
                newdate.append(test)
            elif len(l) == 6:
                test = str(datetime.datetime.strptime(l, '%Y%m'))
                newdate.append(test)
            else:

                test = str(parse(str(l)))
                newdate.append(test)
            # if parse(test) < parse('Feb 18 2015') and version == 2:
            #     raise ValueError(
            #         "GDELT 2.0 only supports \'Feb 18 2015 - "
            #         "Present\'queries currently. Try another date."
            #     )
        # dateString = newdate
        # print("This is the vectorizor datestring {}".format(dateString))
        if version ==1:
            if table != 'gkg':
                # print(base)
                base
                # print("After {}".format(base))

        if not (np.all(list(
                map(
                    lambda x: x > parse('2013 04 01'), list(
                        map(
                            testdate, dateString)))))):

            return (list(
                map(lambda x: base + x + ".zip" if testdate(
                    x).date() < parse(
                    '2013 04 01').date() else base + x + caboose,
                    dateString)))

        else:

            return list(map(lambda x: base + x + caboose, olddateString))

    elif isinstance(dateString, str) is True or len(dateString) == 1:

        if version == 1:
            if table=='events':
                if len(dateString) == 4:
                    comp = datetime.datetime.strptime(dateString,'%Y')
                elif len(dateString) == 6:
                    comp = datetime.datetime.strptime(dateString, '%Y%m')
                else:
                    comp = parse(dateString)
                if comp < parse('2013 Apr 01'):
                    caboose = ".zip"
                elif table == 'events':
                    caboose = ".export.CSV.zip"

        if isinstance(dateString, list) is True or isinstance(
                dateString, np.ndarray) is True:
            dateString = dateString[0]
            if parse(dateString[0]) < parse('2013 Apr 01'):
                caboose = ".zip"

        return base + dateString + caboose

def geofilter(frame):
    """Filters dataframe for conversion to geojson or shapefile"""
    try:
        import geopandas as gpd

        # Remove rows with no latitude and longitude
        try:

            filresults = frame[(frame['ActionGeo_Lat'].notnull()
                                ) | (frame['ActionGeo_Long'].notnull()
                                     )]
        except:

            filresults = frame[(frame['actiongeolat'].notnull()
                                ) | (frame['actiongeolong'].notnull()
                                     )]
        gdf = gpd.GeoDataFrame(filresults.assign(geometry=parallelize_dataframe(filresults)),
                               crs={'init': 'epsg:4326'})
        gdf.columns = list(map(lambda x: (x.replace('_', "")).lower(), gdf.columns))

        # final = gpd.GeoDataFrame(filresults.assign(geometry=parallelize_dataframe(filresults)),
        #                               crs={'init': 'epsg:4326'})

        final = gdf[gdf.geometry.notnull()]

        return final


    except:
        raise ImportError("You need to install geopandas for this feature.")
